﻿using System;
using System.Diagnostics;
using System.Reflection;
using System.ServiceModel;
using System.Windows.Forms;

using WpkgUtilities;

namespace WpkgNotifier
{
    class TrayNotifier : ApplicationContext, IWpkgNotifierMessageClient
    {
        private NotifyIcon notifyIcon;

        private EventLog eventLog;
        private WpkgMessageState state;
        private IWpkgNotifierMessageServer server;
        private int pid;

        private StatusLines statusLines = new StatusLines();
        private WpkgQueryResult LastQueryResult = new WpkgQueryResult();
        private StatusLogDialog statusLogDialog = new StatusLogDialog();
        private StatusDialog statusDialog = new StatusDialog();

        public TrayNotifier()
        {
            Process currentProcess = Process.GetCurrentProcess();
            pid = currentProcess.Id;

            eventLog = new EventLog();
            string eventSourceName = "TrayNotifier";
            string logName = "WpkgNotifier";

            //TODO: need to create the event source at a point where we have admin rights
            if (!EventLog.SourceExists(eventSourceName))
            {
                EventLog.CreateEventSource(eventSourceName, logName);
            }

            eventLog.Source = eventSourceName;
            eventLog.Log = logName;

            statusLogDialog.BindStatusLines(statusLines);
            statusDialog.BindQueryResult(LastQueryResult);

            OpenMessageChannel();

            Application.ApplicationExit += new EventHandler(this.OnApplicationExit);

            MenuItem actionsMenu = new MenuItem("Actions");
            MenuItem syncMenuItem = new MenuItem("Sync", new EventHandler(RequestSync));
            MenuItem queryMenuItem = new MenuItem("Query", new EventHandler(RequestQuery));

            MenuItem statusMenuItem = new MenuItem("Status", new EventHandler(ShowStatus));

            actionsMenu.MenuItems.Add(syncMenuItem);
            actionsMenu.MenuItems.Add(queryMenuItem);

            MenuItem manageSoftwarePage = new MenuItem("Manage software", new EventHandler(ManageSoftware));
            MenuItem exitMenuItem = new MenuItem("Exit", new EventHandler(Exit));

            notifyIcon = new NotifyIcon();
            notifyIcon.Text = "WPKG notifier";
            notifyIcon.Icon = WpkgNotifier.Properties.Resources.wpkgIcon;
            notifyIcon.ContextMenu = new ContextMenu(new MenuItem[]
                { actionsMenu, statusMenuItem, manageSoftwarePage, exitMenuItem });
            notifyIcon.Visible = true;

            // reflectively call our ShowContextMenu method upon left-click
            notifyIcon.MouseDown += (sender, e) =>
            {
                if (e.Button == MouseButtons.Left)
                {
                    MethodInfo mi = typeof(NotifyIcon).GetMethod("ShowContextMenu", BindingFlags.Instance | BindingFlags.NonPublic);
                    if (mi != null)
                    {
                        mi.Invoke(notifyIcon, null);
                    }
                }
            };
        }

        private void OnApplicationExit(object sender, EventArgs e)
        {
            server.Unregister(pid);
        }

        private void OpenMessageChannel()
        {
            eventLog.WriteEntry("Trying to open channel");
            try
            {
                NetNamedPipeBinding binding = new NetNamedPipeBinding();
                string uri = "net.pipe://localhost/www.ch.cam.ac.uk/WpkgNotifier";
                InstanceContext context = new InstanceContext(this);
                DuplexChannelFactory<IWpkgNotifierMessageServer> channelFactory =
                    new DuplexChannelFactory<IWpkgNotifierMessageServer>(context, binding, uri);

                server = channelFactory.CreateChannel();
                eventLog.WriteEntry("channel created");
                server.Register(pid);
                eventLog.WriteEntry("Registered with server");
            }
            catch (Exception e)
            {
                eventLog.WriteEntry("problem creating channel");
                eventLog.WriteEntry(e.Message);
            }
        }

        void RequestSync(object sender, EventArgs e)
        {
            if (statusDialog.Visible)
            {
                statusDialog.Activate();
            }
            else
            {
                statusDialog.ShowDialog();
            }

            // notifyIcon.ContextMenu.MenuItems[0].Enabled = false;
            // server.ProcessRequest(WpkgMessageRequestType.DO_SYNC);
        }

        void RequestQuery(object sender, EventArgs e)
        {
            notifyIcon.ContextMenu.MenuItems[0].Enabled = false;
            server.ProcessRequest(WpkgMessageRequestType.RUN_QUERY);
        }

        void ShowStatus(object sender, EventArgs e)
        {
            if (statusLogDialog.Visible)
            {
                statusLogDialog.Activate();
            }
            else
            {
                statusLogDialog.ShowDialog();
            }
        }

        void ManageSoftware(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start("https://apps.ch.cam.ac.uk/wpkg");
        }

        void Exit(object sender, EventArgs e)
        {
            // We must manually tidy up and remove the icon before we exit.
            // Otherwise it will be left behind until the user mouses over.
            notifyIcon.Visible = false;
            Application.Exit();
        }

        public void SetToolTip(string msg)
        {
            notifyIcon.Text = msg;
        }

        public void HandleResponse(WpkgMessageResponse response)
        {
            eventLog.WriteEntry(response.Message);
        }

        public void SetStatus(WpkgMessageState status)
        {
            this.state = status;
            switch (status)
            {
                case WpkgMessageState.DISCONNECTED:
                    notifyIcon.ContextMenu.MenuItems[0].Enabled = true;
                    notifyIcon.Icon = WpkgNotifier.Properties.Resources.yellow;
                    SetToolTip("Not currently connected");
                    break;
                case WpkgMessageState.IDLE:
                    notifyIcon.ContextMenu.MenuItems[0].Enabled = true;
                    notifyIcon.Icon = WpkgNotifier.Properties.Resources.wpkgIcon;
                    break;
                case WpkgMessageState.SUCCESS:
                    notifyIcon.ContextMenu.MenuItems[0].Enabled = true;
                    notifyIcon.Icon = WpkgNotifier.Properties.Resources.green;
                    break;
                case WpkgMessageState.ERROR:
                    notifyIcon.ContextMenu.MenuItems[0].Enabled = true;
                    notifyIcon.Icon = WpkgNotifier.Properties.Resources.red;
                    break;
                case WpkgMessageState.SYNCING:
                case WpkgMessageState.QUERYING:
                    notifyIcon.ContextMenu.MenuItems[0].Enabled = true;
                    notifyIcon.Icon = WpkgNotifier.Properties.Resources.blue;
                    break;
            }
        }

        public void LogMessage(string msg)
        {
            statusLines.Add(msg);
            eventLog.WriteEntry(msg);
            // SetToolTip(msg);
        }

        public void LogQueryResult(WpkgQueryResult result)
        {
            LastQueryResult.SetResults(result.PendingInstallations, result.PendingUpgrades, result.PendingRemovals);
            eventLog.WriteEntry(String.Format("Query results: {0}", LastQueryResult.StatusText));
        }
    }
}
